@@ -8,12 +8,13 @@
 #include <glib/gi18n-lib.h>
 
 #include <libgnomedb/libgnomedb.h>
+#include <libgnomedb/binreloc/gnome-db-binreloc.h>
 
 #include <demos.h>
 
 /* to be used by example */
-GdaDict       *demo_dict;
 GdaConnection *demo_cnc;
+GdaSqlParser  *demo_parser;
 
 
 static GtkTextBuffer *info_buffer;
@@ -37,30 +38,6 @@ struct _CallbackData
 	GtkTreePath *path;
 };
 
-#ifdef G_OS_WIN32
-
-#undef DEMOCODEDIR
-
-static char *
-get_democodedir (void)
-{
-	static char *result = NULL;
-
-	if (result == NULL) {
-		result = g_win32_get_package_installation_directory (NULL, NULL);
-		if (result == NULL)
-			result = "unknown-location";
-		
-		result = g_strconcat (result, "\\share\\gnome-db\\demo", NULL);
-	}
-
-	return result;
-}
-
-#define DEMOCODEDIR get_democodedir ()
-
-#endif
-
 /**
  * demo_find_file:
  * @base: base filename
@@ -81,7 +58,10 @@ demo_find_file (const char *base, GError **err)
 	    g_file_test (base, G_FILE_TEST_EXISTS))
 		return g_strdup (base);
 	else {
-		char *filename = g_build_filename (DEMOCODEDIR, base, NULL);
+		gchar *filename;
+
+		filename = gnome_db_gbr_get_file_path (GNOME_DB_DATA_DIR, LIBGNOMEDB_ABI_NAME, "demo",
+						       base, NULL);
 		if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
 			g_set_error (err, G_FILE_ERROR, G_FILE_ERROR_NOENT,
 				     "Cannot find demo data file \"%s\"", base);
@@ -755,36 +735,24 @@ main (int argc, char **argv)
 	GtkWidget *hbox;
 	GtkWidget *tree;
 	GtkTextTag *tag;
-	GdaClient *client;
 	GError *error = NULL;
 	gchar *full_filename, *dirname;
 	gchar *cncstring;
 	GtkWidget *msg;
+	gchar *str;
+	GdaMetaStore *mstore;
 
 	/* Initialize i18n support */
-        bindtextdomain (GETTEXT_PACKAGE, LIBGNOMEDB_LOCALEDIR);
+	str = gnome_db_gbr_get_locale_dir_path ();
+        bindtextdomain (GETTEXT_PACKAGE, str);
+	g_free (str);
+
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
 
-        /* Initialize libgnomedb */
-        gnome_db_init ("gnome-db-demo", PACKAGE_VERSION, argc, argv);
-
-	/* Initialize GdaDict object */
-	demo_dict = gda_dict_new ();
-	gda_dict_extend_with_functions (demo_dict);
-	full_filename = demo_find_file ("demo_dict.xml", &error);
-	if (!full_filename) {
-		g_warning (_("Can't find demo dictionary file: %s"), error->message);
-		g_error_free (error);
-		exit (1);
-	}
-	if (!gda_dict_load_xml_file (demo_dict, full_filename, &error)) {
-		g_warning (_("Error loading dictionary from file '%s':\n%s\n"),
-			   full_filename,
-			   error && error->message ?  error->message : _("No detail"));
-		g_error_free (error);
-		exit (1);
-	}
+        /* Initialize libgnomedb & GTK+ */
+        gnome_db_init ();
+	gtk_init (&argc, &argv);
 
 	/* Initialize GdaConnection object */
 	full_filename = demo_find_file ("demo_db.db", &error);
@@ -794,12 +762,10 @@ main (int argc, char **argv)
 		exit (1);
 	}
 
-	client = gda_client_new ();
 	dirname = g_path_get_dirname (full_filename);
 	cncstring = g_strdup_printf ("DB_DIR=%s;DB_NAME=demo_db", dirname);
 	g_free (dirname);
-	demo_cnc = gda_client_open_connection_from_string (client, "SQLite",
-							   cncstring, NULL, NULL, 0, &error);
+	demo_cnc = gda_connection_open_from_string ("SQLite", cncstring, NULL, 0, &error);
 	if (!demo_cnc) {
 		g_warning (_("Error opening the connection for file '%s':\n%s\n"),
 			   full_filename,
@@ -810,8 +776,18 @@ main (int argc, char **argv)
 	g_free (full_filename);
 	g_free (cncstring);
 
-	/* bind the two objects */
-	gda_dict_set_connection (demo_dict, demo_cnc);
+	/* Initialize meta store */
+	full_filename = demo_find_file ("demo_meta.db", &error);
+	mstore = gda_meta_store_new_with_file (full_filename);
+	g_free (full_filename);
+	g_object_set (G_OBJECT (demo_cnc), "meta-store", mstore, NULL);
+        g_object_unref (mstore);
+	/*gda_connection_update_meta_store (demo_cnc, NULL, NULL);*/
+
+	/* Initialize parser object */
+	demo_parser = gda_connection_create_parser (demo_cnc);
+	if (!demo_parser)
+		demo_parser = gda_sql_parser_new ();
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title (GTK_WINDOW (window), _("Gnome-Db Code Demos"));
@@ -872,11 +848,9 @@ main (int argc, char **argv)
 						  _("<b><big>Note:\n</big></b>Many of the demonstrated items use an\n"
 						    "opened connection to the SQLite using the\n"
 						    "'%s' file.\n\n"
-						    "In the source code shown here, "
-						    "The <i>demo_cnc</i> and <i>demo_dict</i>\n"
-						    "objects are created by the framework and "
-						    "made available\n"
-						    "to all the demonstrated items."), full_filename);
+						    "In the source code shown here, the <i>demo_cnc</i> and \n"
+						    "<i>demo_parser</i> objects are created by the framework and\n"
+						    "made available to all the demonstrated items."), full_filename);
 	g_free (full_filename);
 	g_signal_connect_swapped (msg, "response",
 				  G_CALLBACK (gtk_widget_destroy), msg);
