@@ -216,7 +216,7 @@ gnome_db_data_cell_renderer_textual_class_init (GnomeDbDataCellRendererTextualCl
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GnomeDbDataCellRendererTextualClass, changed),
 			      NULL, NULL,
-			      marshal_VOID__STRING_BOXED,
+			      gnome_db_marshal_VOID__STRING_BOXED,
 			      G_TYPE_NONE, 2,
 			      G_TYPE_STRING,
 			      G_TYPE_VALUE);
@@ -402,6 +402,13 @@ gnome_db_data_cell_renderer_textual_set_property (GObject *object,
 	GnomeDbDataCellRendererTextual *datacell = GNOME_DB_DATA_CELL_RENDERER_TEXTUAL (object);
 	gfloat xalign = 0.;
 	const gchar* options;
+	static gchar *too_long_msg = NULL;
+	static gint too_long_msg_len;	
+
+	if (!too_long_msg) {
+		too_long_msg = _("<string cut because too long>");
+		too_long_msg_len = strlen (too_long_msg);
+	}
 
 	switch (param_id) {
 	case PROP_VALUE:
@@ -445,14 +452,10 @@ gnome_db_data_cell_renderer_textual_set_property (GObject *object,
 
 				if (datacell->priv->dh) {
 					str = gda_data_handler_get_str_from_value (datacell->priv->dh, gval);
-					if (strlen (str) > MAX_ACCEPTED_STRING_LENGTH) {
-						gchar *msg;
-						gint len;
-						msg = _("<string cut because too long>");
-						len = strlen (msg);
-						str [MAX_ACCEPTED_STRING_LENGTH] = 0;
-						memmove (str + len, str, MAX_ACCEPTED_STRING_LENGTH - len);
-						memcpy (str, msg, strlen (msg));
+					if (str && (strlen (str) > MAX_ACCEPTED_STRING_LENGTH + too_long_msg_len)) {
+						memmove (str + too_long_msg_len, str, MAX_ACCEPTED_STRING_LENGTH);
+						memcpy (str, too_long_msg, too_long_msg_len);
+						str [MAX_ACCEPTED_STRING_LENGTH + too_long_msg_len + 1] = 0;
 					}
 					
 					if (datacell->priv->options) {
