@@ -2,17 +2,17 @@
  *
  * Copyright (C) 2003 - 2006 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Library General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -94,7 +94,7 @@ gnome_db_entry_wrapper_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbEntryWrapperClass),
 			(GBaseInitFunc) NULL,
@@ -529,7 +529,7 @@ gnome_db_entry_wrapper_set_value_orig (GnomeDbDataEntry *iface, const GValue *va
 		changed = ! mgwrap->priv->real_class->value_is_equal_to (mgwrap, value);
 	else {
 		evalue = gnome_db_entry_wrapper_get_value (iface);
-		if (! gda_value_compare_ext ((GValue *) value, evalue))
+		if (! gda_value_differ ((GValue *) value, evalue))
 			changed = FALSE;
 		if (evalue)
 			gda_value_free (evalue);
@@ -727,6 +727,7 @@ gnome_db_entry_wrapper_get_attributes (GnomeDbDataEntry *iface)
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
 	g_return_val_if_fail (mgwrap->priv, 0);
 
+	check_correct_init (mgwrap);
 	if (!mgwrap->priv->real_class->value_is_equal_to || 
 	    !mgwrap->priv->real_class->value_is_null) {
 		value = gnome_db_entry_wrapper_get_value (iface);
@@ -737,7 +738,7 @@ gnome_db_entry_wrapper_get_attributes (GnomeDbDataEntry *iface)
 	
 	/* NULL? */
 	if (has_current_value) {
-		if (G_VALUE_TYPE (value) == GDA_TYPE_NULL) {
+		if ((value && (G_VALUE_TYPE (value) == GDA_TYPE_NULL)) || !value) {
 			if (mgwrap->priv->default_forced) {
 				if (mgwrap->priv->null_forced)
 					value_is_null = TRUE;
