@@ -2,17 +2,17 @@
  *
  * Copyright (C) 2002 - 2006 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Library General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -21,6 +21,7 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n-lib.h>
+#include "gnome-db-enums.h"
 #include "gnome-db-basic-form.h"
 #include "gnome-db-tools.h"
 #include "marshal.h"
@@ -46,14 +47,14 @@ static void layout_spec_free (GnomeDbFormLayoutSpec *spec);
 static void gnome_db_basic_form_fill (GnomeDbBasicForm *form);
 static void gnome_db_basic_form_clean (GnomeDbBasicForm *form);
 
-static void paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form);
-static void paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form);
-static void paramlist_param_plugin_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form);
-static void paramlist_param_attr_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form);
+static void get_rid_of_set (GdaSet *paramlist, GnomeDbBasicForm *form);
+static void paramlist_public_data_changed_cb (GdaSet *paramlist, GnomeDbBasicForm *form);
+static void paramlist_param_attr_changed_cb (GdaSet *paramlist, GdaHolder *param, 
+					     const gchar *att_name, const GValue *att_value, GnomeDbBasicForm *form);
 
 static void entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form);
 static void entry_contents_activated (GnomeDbDataEntry *entry, GnomeDbBasicForm *form);
-static void parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry);
+static void parameter_changed_cb (GdaHolder *param, GnomeDbDataEntry *entry);
 
 static void mark_not_null_entry_labels (GnomeDbBasicForm *form, gboolean show_mark);
 enum
@@ -76,7 +77,7 @@ enum
 
 struct _GnomeDbBasicFormPriv
 {
-	GdaParameterList       *paramlist;/* parameters */
+	GdaSet                 *set;
 	gulong                 *signal_ids; /* array of signal ids */
 
 	GSList                 *entries;/* list of GnomeDbDataEntry widgets */
@@ -105,7 +106,7 @@ gnome_db_basic_form_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbBasicFormClass),
 			(GBaseInitFunc) NULL,
@@ -138,15 +139,15 @@ gnome_db_basic_form_class_init (GnomeDbBasicFormClass * class)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GnomeDbBasicFormClass, param_changed),
 			      NULL, NULL,
-			      marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE, 2,
-			      GDA_TYPE_PARAMETER, G_TYPE_BOOLEAN);
+			      gnome_db_marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE, 2,
+			      GDA_TYPE_HOLDER, G_TYPE_BOOLEAN);
 	gnome_db_basic_form_signals[ACTIVATED] =
 		g_signal_new ("activated",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GnomeDbBasicFormClass, activated),
 			      NULL, NULL,
-			      marshal_VOID__VOID, G_TYPE_NONE, 0);
+			      gnome_db_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 	class->param_changed = NULL;
 	class->activated = NULL;
@@ -185,7 +186,7 @@ static void
 gnome_db_basic_form_init (GnomeDbBasicForm * wid)
 {
 	wid->priv = g_new0 (GnomeDbBasicFormPriv, 1);
-	wid->priv->paramlist = NULL;
+	wid->priv->set = NULL;
 	wid->priv->entries = NULL;
 	wid->priv->not_null_labels = NULL;
 	wid->priv->layout_spec = NULL;
@@ -208,7 +209,7 @@ static void widget_shown_cb (GtkWidget *wid, GnomeDbBasicForm *form);
 
 /**
  * gnome_db_basic_form_new
- * @paramlist: a #GdaParameterList structure
+ * @paramlist: a #GdaSet structure
  *
  * Creates a new #GnomeDbBasicForm widget using all the parameters provided in @paramlist.
  *
@@ -218,7 +219,7 @@ static void widget_shown_cb (GtkWidget *wid, GnomeDbBasicForm *form);
  * Returns: the new widget
  */
 GtkWidget *
-gnome_db_basic_form_new (GdaParameterList *paramlist)
+gnome_db_basic_form_new (GdaSet *paramlist)
 {
 	GObject *obj;
 
@@ -229,7 +230,7 @@ gnome_db_basic_form_new (GdaParameterList *paramlist)
 
 /**
  * gnome_db_basic_form_new_custom
- * @paramlist: a #GdaParameterList structure
+ * @paramlist: a #GdaSet structure
  * @glade_file: a Glade XML file name
  * @root_element: the name of the top-most widget in @glade_file to use in the new form
  * @form_prefix: the prefix used to look for widgets to add entries in
@@ -242,13 +243,14 @@ gnome_db_basic_form_new (GdaParameterList *paramlist)
  * Returns: the new widget
  */
 GtkWidget *
-gnome_db_basic_form_new_custom (GdaParameterList *paramlist, const gchar *glade_file, 
+gnome_db_basic_form_new_custom (GdaSet *paramlist, const gchar *glade_file, 
 				const gchar *root_element, const gchar *form_prefix)
 {
 	GnomeDbFormLayoutSpec spec;
 	GObject *obj;
-
+#ifdef HAVE_LIBGLADE
 	spec.xml_object = NULL;
+#endif
 	spec.xml_file = (gchar *) glade_file;
 	spec.root_element = (gchar *) root_element;
 	spec.form_prefix = (gchar *) form_prefix;
@@ -272,15 +274,15 @@ widget_shown_cb (GtkWidget *wid, GnomeDbBasicForm *form)
 }
 
 static void
-paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form)
+get_rid_of_set (GdaSet *paramlist, GnomeDbBasicForm *form)
 {
 	GSList *list;
 	gint i = 0;
 
-	g_assert (paramlist == form->priv->paramlist);
+	g_assert (paramlist == form->priv->set);
 
 	/* disconnect from parameters */
-	list = form->priv->paramlist->parameters;;
+	list = form->priv->set->holders;;
 	while (list) {
 		g_signal_handler_disconnect (G_OBJECT (list->data), form->priv->signal_ids[i]);
 		
@@ -291,17 +293,13 @@ paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form)
 	form->priv->signal_ids = NULL;
 
 	/* unref the paramlist */
-	g_signal_handlers_disconnect_by_func (paramlist,
-					      G_CALLBACK (paramlist_destroyed_cb), form);
 	g_signal_handlers_disconnect_by_func (paramlist,
 					      G_CALLBACK (paramlist_public_data_changed_cb), form);
-	g_signal_handlers_disconnect_by_func (paramlist,
-					      G_CALLBACK (paramlist_param_plugin_changed_cb), form);
 	g_signal_handlers_disconnect_by_func (paramlist,
 					      G_CALLBACK (paramlist_param_attr_changed_cb), form);
 
-	g_object_unref (form->priv->paramlist);
-	form->priv->paramlist = NULL;
+	g_object_unref (form->priv->set);
+	form->priv->set = NULL;
 
 	/* render all the entries non sensitive */
 	list = form->priv->entries;
@@ -312,7 +310,7 @@ paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form)
 }
 
 static void
-paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form)
+paramlist_public_data_changed_cb (GdaSet *paramlist, GnomeDbBasicForm *form)
 {
 	/* here we want to re-define all the data entry widgets */
 	gnome_db_basic_form_clean (form);
@@ -320,40 +318,41 @@ paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbBasicForm
 }
 
 static void
-paramlist_param_plugin_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form)
-{
-	/* TODO: be more specific and change only the entry corresponding to @param */
-	paramlist_public_data_changed_cb (paramlist, form);
-}
-
-static void
-paramlist_param_attr_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form)
+paramlist_param_attr_changed_cb (GdaSet *paramlist, GdaHolder *param, 
+				 const gchar *att_name, const GValue *att_value, GnomeDbBasicForm *form)
 {
 	GtkWidget *entry;
 
-	entry = gnome_db_basic_form_get_entry_widget (form, param);
-	if (entry) {
-		gboolean toset;
-		guint attrs = 0;
-		guint mask = 0;
-
-		toset = gda_parameter_get_exists_default_value (param);
-		attrs |= toset ? GDA_VALUE_ATTR_CAN_BE_DEFAULT : 0;
-		mask |= GDA_VALUE_ATTR_CAN_BE_DEFAULT;
-
-		toset = gda_parameter_get_not_null (param);
-		attrs |= toset ? 0 : GDA_VALUE_ATTR_CAN_BE_NULL;
-		mask |= GDA_VALUE_ATTR_CAN_BE_NULL;
-
-		g_object_get (G_OBJECT (param), "use-default-value", &toset, NULL);
-		attrs |= toset ? GDA_VALUE_ATTR_IS_DEFAULT : 0;
-		mask |= GDA_VALUE_ATTR_IS_DEFAULT;
-
-		g_signal_handlers_block_by_func (G_OBJECT (entry),
-						 G_CALLBACK (entry_contents_modified), form);
-		gnome_db_data_entry_set_attributes (GNOME_DB_DATA_ENTRY (entry), attrs, mask);
-		g_signal_handlers_unblock_by_func (G_OBJECT (entry),
-						   G_CALLBACK (entry_contents_modified), form);
+	if (!strcmp (att_name, GDA_ATTRIBUTE_IS_DEFAULT)) {
+		entry = gnome_db_basic_form_get_entry_widget (form, param);
+		if (entry) {
+			gboolean toset;
+			guint attrs = 0;
+			guint mask = 0;
+			const GValue *defv;
+			
+			defv = gda_holder_get_default_value (param);
+			attrs |= defv ? GDA_VALUE_ATTR_CAN_BE_DEFAULT : 0;
+			mask |= GDA_VALUE_ATTR_CAN_BE_DEFAULT;
+			
+			toset = gda_holder_get_not_null (param);
+			attrs |= toset ? 0 : GDA_VALUE_ATTR_CAN_BE_NULL;
+			mask |= GDA_VALUE_ATTR_CAN_BE_NULL;
+			
+			g_object_get (G_OBJECT (param), "use-default-value", &toset, NULL);
+			attrs |= toset ? GDA_VALUE_ATTR_IS_DEFAULT : 0;
+			mask |= GDA_VALUE_ATTR_IS_DEFAULT;
+			
+			g_signal_handlers_block_by_func (G_OBJECT (entry),
+							 G_CALLBACK (entry_contents_modified), form);
+			gnome_db_data_entry_set_attributes (GNOME_DB_DATA_ENTRY (entry), attrs, mask);
+			g_signal_handlers_unblock_by_func (G_OBJECT (entry),
+							   G_CALLBACK (entry_contents_modified), form);
+		}
+	}
+	else if (!strcmp (att_name, GNOME_DB_ATTRIBUTE_PLUGIN)) {
+		/* TODO: be more specific and change only the cell renderer corresponding to @param */
+		paramlist_public_data_changed_cb (paramlist, form);
 	}
 }
 
@@ -368,8 +367,8 @@ gnome_db_basic_form_dispose (GObject *object)
 
 	if (form->priv) {
 		/* paramlist */
-		if (form->priv->paramlist) 
-			paramlist_destroyed_cb (form->priv->paramlist, form);
+		if (form->priv->set) 
+			get_rid_of_set (form->priv->set, form);
 
 		gnome_db_basic_form_clean (form);
 
@@ -391,12 +390,15 @@ gnome_db_basic_form_set_property (GObject *object,
 				  GParamSpec *pspec)
 {
 	GnomeDbBasicForm *form;
+#ifdef HAVE_LIBGLADE
 	GnomeDbFormLayoutSpec *lspec, *new_spec = NULL;
+#endif
 
         form = GNOME_DB_BASIC_FORM (object);
         if (form->priv) {
                 switch (param_id) {
 		case PROP_LAYOUT_SPEC:
+#ifdef HAVE_LIBGLADE
 			lspec = g_value_get_pointer (value);
 			if (lspec) {
 				g_return_if_fail (lspec->xml_file || lspec->xml_object);
@@ -432,36 +434,69 @@ gnome_db_basic_form_set_property (GObject *object,
 				g_print ("Loaded Glade file, reinit interface\n");
 			}
 			gnome_db_basic_form_fill (form);
+#else
+			g_warning (_("Libglade support not built."));
+#endif
 			break;
 		case PROP_PARAMLIST:
-			if (form->priv->paramlist) {
-				paramlist_destroyed_cb (form->priv->paramlist, form);
+			if (form->priv->set) {
+#ifdef HAVE_LIBGLADE
+			new_spec = NULL;
+			if (form->priv->layout_spec) {
+				/* old spec */
+				lspec = form->priv->layout_spec;
+				/* spec copy */
+				new_spec = g_new0 (GnomeDbFormLayoutSpec, 1);
+				if (lspec->xml_file)
+					new_spec->xml_file = g_strdup (lspec->xml_file);
+				if (lspec->xml_object) {
+					new_spec->xml_object = lspec->xml_object;
+					g_object_ref (new_spec->xml_object);
+				}
+				if (lspec->root_element)
+					new_spec->root_element = g_strdup (lspec->root_element);
+				if (lspec->form_prefix)
+					new_spec->form_prefix = g_strdup (lspec->form_prefix);
+			}
+#endif
+#ifdef HAVE_LIBGLADE
+			new_spec = NULL;
+			if (form->priv->layout_spec) {
+				/* old spec */
+				lspec = form->priv->layout_spec;
+				/* spec copy */
+				new_spec = g_new0 (GnomeDbFormLayoutSpec, 1);
+				if (lspec->xml_file)
+					new_spec->xml_file = g_strdup (lspec->xml_file);
+				if (lspec->xml_object) {
+					new_spec->xml_object = lspec->xml_object;
+					g_object_ref (new_spec->xml_object);
+				}
+				if (lspec->root_element)
+					new_spec->root_element = g_strdup (lspec->root_element);
+				if (lspec->form_prefix)
+					new_spec->form_prefix = g_strdup (lspec->form_prefix);
+			}
+#endif
+				get_rid_of_set (form->priv->set, form);
 				gnome_db_basic_form_clean (form);
 			}
 
-			form->priv->paramlist = g_value_get_pointer (value);
-			if (form->priv->paramlist) {
-				GError *error = NULL;
-				g_return_if_fail (GDA_IS_PARAMETER_LIST (form->priv->paramlist));
-				
-				if (! gda_parameter_list_is_coherent (form->priv->paramlist, &error)) {
-					g_warning ("gda_parameter_list_is_coherent() returned FALSE: %s", 
-						   error->message);
-					form->priv->paramlist = NULL;
-					g_error_free (error);
-					return;
-				}
-				
-				g_object_ref (form->priv->paramlist);
-				gda_object_connect_destroy (form->priv->paramlist,
-							    G_CALLBACK (paramlist_destroyed_cb), form);
-				g_signal_connect (form->priv->paramlist, "public_data_changed",
+			form->priv->set = g_value_get_pointer (value);
+			if (form->priv->set) {
+				g_return_if_fail (GDA_IS_SET (form->priv->set));
+								
+				g_object_ref (form->priv->set);
+				g_signal_connect (form->priv->set, "public_data_changed",
 						  G_CALLBACK (paramlist_public_data_changed_cb), form);
-				g_signal_connect (form->priv->paramlist, "param_plugin_changed",
-						  G_CALLBACK (paramlist_param_plugin_changed_cb), form);
-				g_signal_connect (form->priv->paramlist, "param_attr_changed",
+				g_signal_connect (form->priv->set, "holder-attr-changed",
 						  G_CALLBACK (paramlist_param_attr_changed_cb), form);
-				
+
+#ifdef HAVE_LIBGLADE
+				if (new_spec)
+					form->priv->layout_spec = new_spec;
+				new_spec = NULL;
+#endif
 				gnome_db_basic_form_fill (form);
 			}
 			break;
@@ -493,7 +528,7 @@ gnome_db_basic_form_get_property (GObject *object,
         if (form->priv) {
                 switch (param_id) {
 		case PROP_PARAMLIST:
-			g_value_set_pointer (value, form->priv->paramlist);
+			g_value_set_pointer (value, form->priv->set);
 			break;
 		case PROP_HEADERS_SENSITIVE:
 			g_value_set_boolean (value, form->priv->headers_sensitive);
@@ -517,8 +552,8 @@ gnome_db_basic_form_clean (GnomeDbBasicForm *form)
 	GSList *list;
 	gint i = 0;
 
-	if (form->priv->paramlist) {
-		list = form->priv->paramlist->parameters;
+	if (form->priv->set) {
+		list = form->priv->set->holders;
 		while (list) {
 			g_signal_handler_disconnect (G_OBJECT (list->data), form->priv->signal_ids[i]);
 			
@@ -557,8 +592,10 @@ gnome_db_basic_form_clean (GnomeDbBasicForm *form)
 static void
 layout_spec_free (GnomeDbFormLayoutSpec *spec)
 {
+#ifdef HAVE_LIBGLADE
 	if (spec->xml_object)
 		g_object_unref (spec->xml_object);
+#endif
 	g_free (spec->xml_file);
 	g_free (spec->root_element);
 	g_free (spec->form_prefix);
@@ -577,54 +614,57 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 	GSList *list;
 	gint i;
 	gboolean form_expand = FALSE;
-	GdaDict *dict;
 	
 	/* parameters list management */
-	if (!form->priv->paramlist || !form->priv->paramlist->groups_list)
+	if (!form->priv->set || !form->priv->set->groups_list)
 		/* nothing to do */
 		return;
 
-	dict = gda_object_get_dict (GDA_OBJECT (form->priv->paramlist));
-
 	/* allocating space for the signal ids and connect to the parameter's changes */
-	form->priv->signal_ids = g_new0 (gulong, g_slist_length (form->priv->paramlist->parameters));
+	form->priv->signal_ids = g_new0 (gulong, g_slist_length (form->priv->set->holders));
 	i = 0;
 
 	/* creating all the data entries, and putting them into the form->priv->entries list */
-	list = form->priv->paramlist->groups_list;
+	list = form->priv->set->groups_list;
 	while (list) {
-		GdaParameterListGroup *group;
+		GdaSetGroup *group;
 		GtkWidget *entry = NULL;
 
-		group = GDA_PARAMETER_LIST_GROUP (list->data);
+		group = GDA_SET_GROUP (list->data);
 		if (! group->nodes_source) { 
 			/* there is only one non-constrained parameter */
-			GdaParameter *param;
+			GdaHolder *param;
 			GType type;
 			const GValue *val, *default_val, *value;
 			gboolean nnul;
-			gchar *plugin = NULL;
+			const gchar *plugin = NULL;
+			const GValue *plugin_val;
 
 			g_assert (g_slist_length (group->nodes) == 1);
 
-			param = GDA_PARAMETER (GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
+			param = GDA_HOLDER (GDA_SET_NODE (group->nodes->data)->holder);
 
-			val = gda_parameter_get_value (param);
-			default_val = gda_parameter_get_default_value (param);
-			nnul = gda_parameter_get_not_null (param);
+			val = gda_holder_get_value (param);
+			default_val = gda_holder_get_default_value (param);
+			nnul = gda_holder_get_not_null (param);
 
 			/* determine initial value */
-			type = gda_parameter_get_g_type (param);
+			type = gda_holder_get_g_type (param);
 			value = val;
 			if (!value && default_val && 
 			    (G_VALUE_TYPE ((GValue *) default_val) == type))
 				value = default_val;
 			
 			/* create entry */
-			g_object_get (G_OBJECT (param), "entry_plugin", &plugin, NULL);
-			entry = GTK_WIDGET (gnome_db_util_new_data_entry (dict, type, plugin));
-			g_free(plugin);
-
+			plugin_val = gda_holder_get_attribute (param, GNOME_DB_ATTRIBUTE_PLUGIN);
+			if (plugin_val) {
+				if (G_VALUE_TYPE (plugin_val) == G_TYPE_STRING)
+					plugin = g_value_get_string (plugin_val);
+				else
+					g_warning (_("The '%s' attribute should be a G_TYPE_STRING value"),
+						   GNOME_DB_ATTRIBUTE_PLUGIN);
+			}
+			entry = GTK_WIDGET (gnome_db_util_new_data_entry (type, plugin));
 
 			/* set current value */
 			gnome_db_data_entry_set_value (GNOME_DB_DATA_ENTRY (entry), val);
@@ -661,25 +701,23 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 			GSList *plist;
 			gboolean nnul = TRUE;
 
-			entry = gnome_db_entry_combo_new (form->priv->paramlist, group->nodes_source);
+			entry = gnome_db_entry_combo_new (form->priv->set, group->nodes_source);
 			g_object_set_data (G_OBJECT (entry), "group", group);
 			g_object_set_data (G_OBJECT (entry), "form", form);
 			form->priv->entries = g_slist_append (form->priv->entries, entry);
 			g_signal_connect (entry, "destroy", G_CALLBACK (entry_destroyed_cb), form);
 
 			/* connect to the parameter's changes */
-			plist = group->nodes;
-			while (plist) {
-				GdaParameter *param;
+			for (plist = group->nodes; plist; plist = plist->next) {
+				GdaHolder *param;
 
-				param = GDA_PARAMETER_LIST_NODE (plist->data)->param;
-				if (!gda_parameter_get_not_null (param))
+				param = GDA_SET_NODE (plist->data)->holder;
+				if (!gda_holder_get_not_null (param))
 					nnul = FALSE;
 				form->priv->signal_ids[i] = g_signal_connect (param, "changed",
                                                                               G_CALLBACK (parameter_changed_cb), 
 									      entry);
                                 i++;
-				plist = g_slist_next (plist);
 			}
 			gnome_db_data_entry_set_attributes (GNOME_DB_DATA_ENTRY (entry),
 							    nnul ? 0 : GDA_VALUE_ATTR_CAN_BE_NULL,
@@ -698,6 +736,7 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 	/*
 	 * If there is a layout spec, then try to use it
 	 */
+#ifdef HAVE_LIBGLADE
 	if (form->priv->layout_spec) {
 		GtkWidget *layout = NULL;
 		
@@ -715,13 +754,13 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 			
 			gtk_box_pack_start (GTK_BOX (form), layout,  TRUE, TRUE, 0);
 			list = form->priv->entries;
-			groups = form->priv->paramlist->groups_list;
+			groups = form->priv->set->groups_list;
 			while (groups && list) {
 				gint param_no;
 				gchar *box_name;
 
-				param_no = g_slist_index (form->priv->paramlist->parameters,
-							  ((GdaParameterListNode *)(((GdaParameterListGroup *)groups->data)->nodes->data))->param);
+				param_no = g_slist_index (form->priv->set->holders,
+							  ((GdaSetNode *)(((GdaSetGroup *)groups->data)->nodes->data))->holder);
 				box_name = g_strdup_printf ("%s_%d", form->priv->layout_spec->form_prefix, param_no);
 				box = glade_xml_get_widget (form->priv->layout_spec->xml_object, box_name);
 				g_print ("Box named %s => %p\n", box_name, box);
@@ -744,6 +783,7 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 			gtk_widget_show (layout);
 		}
 	}
+#endif
 
 	/* 
 	 * There is no layout spec (or the provided one could not be used),
@@ -763,26 +803,31 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 		while (list) {
 			gboolean expand;
 			GtkWidget *entry_label;
-			GdaParameter *param;
+			GdaHolder *param;
 			
 			/* label for the entry */
 			param = g_object_get_data (G_OBJECT (list->data), "param");
 			if (param) {
-				const gchar *title;
+				gchar *title;
 				gchar *str;
 				GtkWidget *evbox;
 
-				title = gda_object_get_name (GDA_OBJECT (param));
+				g_object_get (G_OBJECT (param), "name", &title, NULL);
 				if (!title)
-					title = _("Value");
+					title = g_strdup (_("Value"));
 				evbox = gtk_event_box_new ();
 				str = g_strdup_printf ("%s:", title);
 				label = gtk_label_new (str);
 				g_free (str);
-				g_object_set_data_full (G_OBJECT (label), "_gda_title", g_strdup (title), g_free);
-				if (gda_parameter_get_not_null (param)) 
+				g_object_set_data_full (G_OBJECT (label), "_gda_title", title, g_free);
+				if (gda_holder_get_not_null (param)) 
 					form->priv->not_null_labels = g_slist_prepend (form->priv->not_null_labels, label);
-				
+
+#ifdef HAVE_LIBGLADE_FIXME
+				if (new_spec)
+					form->priv->layout_spec = new_spec;
+				new_spec = NULL;
+#endif
 				gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 				gtk_container_add (GTK_CONTAINER (evbox), label);
 				
@@ -792,38 +837,42 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 				gtk_widget_show (label);
 				entry_label = evbox;
 				
-				title = gda_object_get_description (GDA_OBJECT (param));
+				g_object_get (G_OBJECT (param), "description", &title, NULL);
 				if (title && *title)
 					gtk_tooltips_set_tip (form->priv->tooltips, evbox, title, NULL);
+				g_free (title);
 			}
 			else {
 				/* FIXME: find a better label and tooltip and improve data entry attributes */
-				const gchar *title = NULL;
+				gchar *title = NULL;
 				gchar *str;
 				GtkWidget *evbox;
 				gboolean nullok = TRUE;
 				GSList *params;
-				GdaParameterListGroup *group;
+				GdaSetGroup *group;
 
 				group = g_object_get_data (G_OBJECT (list->data), "group");
-				params = group->nodes;
-				while (params) {
-					if (nullok && gda_parameter_get_not_null (GDA_PARAMETER_LIST_NODE (params->data)->param))
+				for (params = group->nodes; params; params = params->next) {
+					if (nullok && gda_holder_get_not_null (GDA_SET_NODE (params->data)->holder))
 						nullok = FALSE;
 					if (!title)
-						title = gda_object_get_name (GDA_OBJECT (GDA_PARAMETER_LIST_NODE (params->data)->param));
-					params = g_slist_next (params);
+						g_object_get (G_OBJECT (GDA_SET_NODE (params->data)->holder), 
+							      "name", &title, NULL);
 				}
 				
+				if (!title) {
+					str = g_object_get_data (G_OBJECT (group->nodes_source->data_model), 
+								 "name");
+					if (str)
+						title = g_strdup (str);
+				}
 				if (!title)
-					title = gda_object_get_name (GDA_OBJECT (group->nodes_source->data_model));
-				if (!title)
-					title = _("Value");
+					title = g_strdup (_("Value"));
 				evbox = gtk_event_box_new ();
 				str = g_strdup_printf ("%s:", title);
 				label = gtk_label_new (str);
 				g_free (str);
-				g_object_set_data_full (G_OBJECT (label), "_gda_title", g_strdup (title), g_free);
+				g_object_set_data_full (G_OBJECT (label), "_gda_title", title, g_free);
 				if (!nullok) 
 					form->priv->not_null_labels = g_slist_prepend (form->priv->not_null_labels, label);
 				gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
@@ -835,7 +884,7 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 				gtk_widget_show (label);
 				entry_label = evbox;
 				
-				title = gda_object_get_description (GDA_OBJECT (group->nodes_source->data_model));
+				title = g_object_get_data (G_OBJECT (group->nodes_source->data_model), "descr");
 				if (title && *title)
 					gtk_tooltips_set_tip (form->priv->tooltips, evbox, title, NULL);
 			}
@@ -870,7 +919,7 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 	/* hide some entries if specified */
 	for (list = form->priv->entries; list; list = list->next) {
 		gboolean explicit_hide = FALSE;
-		GdaParameter *param;
+		GdaHolder *param;
 			
 		param = g_object_get_data (G_OBJECT (list->data), "param");
 		if (param) {
@@ -952,7 +1001,7 @@ entry_contents_activated (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 static void
 entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 {
-	GdaParameter *param;
+	GdaHolder *param;
 	GdaValueAttribute attr;
 
 	attr = gnome_db_data_entry_get_attributes (entry);
@@ -960,30 +1009,30 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 	if (param) { /* single parameter */
 		GValue *value;
 		
-		form->priv->forward_param_updates = FALSE;
+		form->priv->forward_param_updates = TRUE;
 
 		/* parameter's value */
 		value = gnome_db_data_entry_get_value (entry);
 		if ((!value || gda_value_is_null (value)) &&
 		    (attr & GDA_VALUE_ATTR_IS_DEFAULT))
-			g_object_set (G_OBJECT (param), "use_default_value", TRUE, NULL);
+			gda_holder_set_value_to_default (param);
+		else if (gda_holder_set_value (param, value, NULL)) {
+#ifdef debug_signal
+			g_print (">> 'PARAM_CHANGED' from %s\n", __FUNCTION__);
+#endif
+			g_signal_emit (G_OBJECT (form), gnome_db_basic_form_signals[PARAM_CHANGED], 0, param, TRUE);
+#ifdef debug_signal
+			g_print ("<< 'PARAM_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		}
 		else
-			g_object_set (G_OBJECT (param), "use_default_value", FALSE, NULL);
-		gda_parameter_set_value (param, value);
-#ifdef debug_signal
-		g_print (">> 'PARAM_CHANGED' from %s\n", __FUNCTION__);
-#endif
-		g_signal_emit (G_OBJECT (form), gnome_db_basic_form_signals[PARAM_CHANGED], 0, param, TRUE);
-#ifdef debug_signal
-		g_print ("<< 'PARAM_CHANGED' from %s\n", __FUNCTION__);
-#endif
-		form->priv->forward_param_updates = TRUE;
+			TO_IMPLEMENT;
 		gda_value_free (value);
 	}
 	else { /* multiple parameters */
 		GSList *params;
 		GSList *values, *list;
-		GdaParameterListGroup *group;
+		GdaSetGroup *group;
 
 		group = g_object_get_data (G_OBJECT (entry), "group");
 		params = group->nodes;
@@ -1001,21 +1050,24 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 			 * - modify the signal to add a list of parameters which are changed 
 			 *   and emit only one signal.
 			 */
-			GdaParameter *param;
+			GdaHolder *param;
 			form->priv->forward_param_updates = FALSE;
 
 			/* parameter's value */
-			param = GDA_PARAMETER_LIST_NODE (params->data)->param;
-			gda_parameter_set_value (param, (GValue *)(list->data));
+			param = GDA_SET_NODE (params->data)->holder;
+			if (gda_holder_set_value (param, (GValue *)(list->data), NULL)) {
 #ifdef debug_signal
-			g_print (">> 'PARAM_CHANGED' from %s\n", __FUNCTION__);
+				g_print (">> 'PARAM_CHANGED' from %s\n", __FUNCTION__);
 #endif
-			g_signal_emit (G_OBJECT (form), gnome_db_basic_form_signals[PARAM_CHANGED], 
-				       0, param, TRUE);
+				g_signal_emit (G_OBJECT (form), gnome_db_basic_form_signals[PARAM_CHANGED], 
+					       0, param, TRUE);
 #ifdef debug_signal
-			g_print ("<< 'PARAM_CHANGED' from %s\n", __FUNCTION__);
+				g_print ("<< 'PARAM_CHANGED' from %s\n", __FUNCTION__);
 #endif
-			form->priv->forward_param_updates = TRUE;;
+			}
+			else
+				TO_IMPLEMENT;
+			form->priv->forward_param_updates = TRUE;
 
 			list = g_slist_next (list);
 			params = g_slist_next (params);
@@ -1024,13 +1076,13 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 
 #ifdef PROXY_STORE_EXTRA_VALUES
 		/* updating the GdaDataProxy if there is one */
-		if (GDA_IS_DATA_MODEL_ITER (form->priv->paramlist)) {
+		if (GDA_IS_DATA_MODEL_ITER (form->priv->set)) {
 			GdaDataProxy *proxy;
 			gint proxy_row;
 			
-			proxy_row = gda_data_model_iter_get_row ((GdaDataModelIter *) form->priv->paramlist);
+			proxy_row = gda_data_model_iter_get_row ((GdaDataModelIter *) form->priv->set);
 
-			g_object_get (G_OBJECT (form->priv->paramlist), "data_model", &proxy, NULL);
+			g_object_get (G_OBJECT (form->priv->set), "data_model", &proxy, NULL);
 			if (GDA_IS_DATA_PROXY (proxy)) {
 				GSList *all_new_values;
 				gint i, col;
@@ -1060,11 +1112,11 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
  * the param change does not come from a GnomeDbDataEntry change.
  */ 
 static void
-parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
+parameter_changed_cb (GdaHolder *param, GnomeDbDataEntry *entry)
 {
 	GnomeDbBasicForm *form = g_object_get_data (G_OBJECT (entry), "form");
-	GdaParameterListGroup *group = g_object_get_data (G_OBJECT (entry), "group");
-	const GValue *value = gda_parameter_get_value (param);
+	GdaSetGroup *group = g_object_get_data (G_OBJECT (entry), "group");
+	const GValue *value = gda_holder_get_value (param);
 
 	if (form->priv->forward_param_updates) {
 		gboolean param_valid;
@@ -1072,7 +1124,7 @@ parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
 
 		/* There can be a feedback from the entry if the param is invalid and "set_default_if_invalid"
 		   exists and is TRUE */
-		param_valid = gda_parameter_is_valid (param);
+		param_valid = gda_holder_is_valid (param);
 		if (!param_valid) 
 			if (g_object_class_find_property (G_OBJECT_GET_CLASS (entry), "set_default_if_invalid"))
 				g_object_get (G_OBJECT (entry), "set_default_if_invalid", &default_if_invalid, NULL);
@@ -1091,7 +1143,7 @@ parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
 
 			while (list) {
 				const GValue *pvalue;
-				pvalue = gda_parameter_get_value (GDA_PARAMETER_LIST_NODE (list->data)->param);
+				pvalue = gda_holder_get_value (GDA_SET_NODE (list->data)->holder);
 				values = g_slist_append (values, (GValue *) pvalue);
 				if (allnull && pvalue && 
 				    (G_VALUE_TYPE ((GValue *) pvalue) != GDA_TYPE_NULL))
@@ -1131,18 +1183,18 @@ parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
  * gnome_db_basic_form_get_paramlist
  * @form: a #GnomeDbBasicForm widget
  *
- * Get a pointer to the #GdaParameterList used internally by @form to store
+ * Get a pointer to the #GdaSet used internally by @form to store
  * values
  *
  * Returns:
  */
-GdaParameterList *
+GdaSet *
 gnome_db_basic_form_get_paramlist (GnomeDbBasicForm *form)
 {
 	g_return_val_if_fail (form && GNOME_DB_IS_BASIC_FORM (form), NULL);
 	g_return_val_if_fail (form->priv, NULL);
 
-	return form->priv->paramlist;
+	return form->priv->set;
 }
 
 /**
@@ -1158,14 +1210,14 @@ void
 gnome_db_basic_form_set_current_as_orig (GnomeDbBasicForm *form)
 {
 	GSList *list;
-	GdaParameter *param;
+	GdaHolder *param;
 
 	g_return_if_fail (form && GNOME_DB_IS_BASIC_FORM (form));
 	g_return_if_fail (form->priv);
 
 	list = form->priv->entries;
 	while (list) {
-		GdaParameterListGroup *group;
+		GdaSetGroup *group;
 
 		group = g_object_get_data (G_OBJECT (list->data), "group");
 
@@ -1177,7 +1229,7 @@ gnome_db_basic_form_set_current_as_orig (GnomeDbBasicForm *form)
 			
 			while (params) {
 				const GValue *pvalue;
-				pvalue = gda_parameter_get_value (GDA_PARAMETER_LIST_NODE (params->data)->param);
+				pvalue = gda_holder_get_value (GDA_SET_NODE (params->data)->holder);
 				values = g_slist_append (values, (GValue *) pvalue);
 				if (allnull && pvalue && 
 				    (G_VALUE_TYPE ((GValue *) pvalue) != GDA_TYPE_NULL))
@@ -1196,7 +1248,7 @@ gnome_db_basic_form_set_current_as_orig (GnomeDbBasicForm *form)
 		else {
 			/* non combo entry */
 			param = g_object_get_data (G_OBJECT (list->data), "param");
-			gnome_db_data_entry_set_value_orig (GNOME_DB_DATA_ENTRY (list->data), gda_parameter_get_value (param));
+			gnome_db_data_entry_set_value_orig (GNOME_DB_DATA_ENTRY (list->data), gda_holder_get_value (param));
 		}
 		list = g_slist_next (list);
 	}
@@ -1216,25 +1268,25 @@ gnome_db_basic_form_is_valid (GnomeDbBasicForm *form)
 	g_return_val_if_fail (form && GNOME_DB_IS_BASIC_FORM (form), FALSE);
 	g_return_val_if_fail (form->priv, FALSE);
 
-	return gda_parameter_list_is_valid (form->priv->paramlist);
+	return gda_set_is_valid (form->priv->set, NULL);
 }
 
 /**
  * gnome_db_basic_form_get_data_set
  * @form: a #GnomeDbBasicForm widget
  *
- * Get a pointer to the #GdaParameterList object which
+ * Get a pointer to the #GdaSet object which
  * is modified by @form
  *
  * Returns:
  */
-GdaParameterList *
+GdaSet *
 gnome_db_basic_form_get_data_set (GnomeDbBasicForm *form)
 {
 	g_return_val_if_fail (form && GNOME_DB_IS_BASIC_FORM (form), NULL);
 	g_return_val_if_fail (form->priv, NULL);
 
-	return form->priv->paramlist;
+	return form->priv->set;
 }
 
 /**
@@ -1311,7 +1363,7 @@ gnome_db_basic_form_reset (GnomeDbBasicForm *form)
 	
 	list = form->priv->entries;
 	while (list) {
-		GdaParameterListNode *node = g_object_get_data (G_OBJECT (list->data), "node");
+		GdaSetNode *node = g_object_get_data (G_OBJECT (list->data), "node");
 
 		if (node) {
 			/* Combo entry */
@@ -1336,14 +1388,14 @@ gnome_db_basic_form_reset (GnomeDbBasicForm *form)
 /**
  * gnome_db_basic_form_entry_show
  * @form: a #GnomeDbBasicForm widget
- * @param: a #GdaParameter object
+ * @param: a #GdaHolder object
  * @show:
  *
  * Shows or hides the #GnomeDbDataEntry in @form which corresponds to the
  * @param parameter
  */
 void
-gnome_db_basic_form_entry_show (GnomeDbBasicForm *form, GdaParameter *param, gboolean show)
+gnome_db_basic_form_entry_show (GnomeDbBasicForm *form, GdaHolder *param, gboolean show)
 {
 	GtkWidget *entry;
 
@@ -1405,12 +1457,12 @@ gnome_db_basic_form_entry_show (GnomeDbBasicForm *form, GdaParameter *param, gbo
 /**
  * gnome_db_basic_form_entry_grab_focus
  * @form: a #GnomeDbBasicForm widget
- * @param: a #GdaParameter object
+ * @param: a #GdaHolder object
  * 
  * Makes the data entry corresponding to @param grab the focus for the window it's in
  */
 void
-gnome_db_basic_form_entry_grab_focus (GnomeDbBasicForm *form, GdaParameter *param)
+gnome_db_basic_form_entry_grab_focus (GnomeDbBasicForm *form, GdaHolder *param)
 {
 	GtkWidget *entry;
 
@@ -1426,14 +1478,14 @@ gnome_db_basic_form_entry_grab_focus (GnomeDbBasicForm *form, GdaParameter *para
 /**
  * gnome_db_basic_form_entry_set_editable
  * @form: a #GnomeDbBasicForm widget
- * @param: a #GdaParameter object
+ * @param: a #GdaHolder object
  * @editable:
  *
  * Sets the #GnomeDbDataEntry in @form which corresponds to the
  * @param parameter editable or not
  */
 void
-gnome_db_basic_form_entry_set_editable (GnomeDbBasicForm *form, GdaParameter *param, gboolean editable)
+gnome_db_basic_form_entry_set_editable (GnomeDbBasicForm *form, GdaHolder *param, gboolean editable)
 {
 	GtkWidget *entry;
 
@@ -1505,19 +1557,19 @@ gnome_db_basic_form_set_entries_default (GnomeDbBasicForm *form)
 	}
 }
 
-static void form_param_changed (GnomeDbBasicForm *form, GdaParameter *param, gboolean is_user_modif, GtkDialog *dlg);
+static void form_param_changed (GnomeDbBasicForm *form, GdaHolder *param, gboolean is_user_modif, GtkDialog *dlg);
 
 /**
  * gnome_db_basic_form_get_entry_widget
  * @form: a #GnomeDbBasicForm widget
- * @param: a #GdaParameter object
+ * @param: a #GdaHolder object
  *
  * Get the #GnomeDbDataEntry in @form which corresponds to the param parameter.
  *
  * Returns: the requested widget, or %NULL if not found
  */
 GtkWidget *
-gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaParameter *param)
+gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaHolder *param)
 {
 	GSList *entries;
 	GtkWidget *entry = NULL;
@@ -1527,7 +1579,7 @@ gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaParameter *para
 
 	entries = form->priv->entries;
 	while (entries && !entry) {
-		GdaParameter *thisparam = g_object_get_data (G_OBJECT (entries->data), "param");
+		GdaHolder *thisparam = g_object_get_data (G_OBJECT (entries->data), "param");
 
 		if (thisparam) {
 			if (thisparam == param)
@@ -1536,12 +1588,12 @@ gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaParameter *para
 		else {
 			/* multiple parameters */
 			GSList *params;
-			GdaParameterListGroup *group;
+			GdaSetGroup *group;
 
 			group = g_object_get_data (G_OBJECT (entries->data), "group");
 			params = group->nodes;
 			while (params && !entry) {
-				if (GDA_PARAMETER_LIST_NODE (params->data)->param == (gpointer) param)
+				if (GDA_SET_NODE (params->data)->holder == (gpointer) param)
 					entry = GTK_WIDGET (entries->data);
 				params = g_slist_next (params);
 			}
@@ -1556,14 +1608,14 @@ gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaParameter *para
 /**
  * gnome_db_basic_form_get_label_widget
  * @form: a #GnomeDbBasicForm widget
- * @param: a #GdaParameter object
+ * @param: a #GdaHolder object
  *
  * Get the label in @form which corresponds to the param parameter.
  *
  * Returns: the requested widget, or %NULL if not found
  */
 GtkWidget *
-gnome_db_basic_form_get_label_widget (GnomeDbBasicForm *form, GdaParameter *param)
+gnome_db_basic_form_get_label_widget (GnomeDbBasicForm *form, GdaHolder *param)
 {
 	GtkWidget *entry;
 
@@ -1580,8 +1632,7 @@ gnome_db_basic_form_get_label_widget (GnomeDbBasicForm *form, GdaParameter *para
 
 /**
  * gnome_db_basic_form_new_in_dialog
- * @dict: a #GdaDict object
- * @paramlist: a #GdaParameterList structure
+ * @paramlist: a #GdaSet structure
  * @parent: the parent window for the new dialog, or %NULL
  * @title: the title of the dialog window, or %NULL
  * @header: a helper text displayed at the top of the dialog, or %NULL
@@ -1596,7 +1647,7 @@ gnome_db_basic_form_get_label_widget (GnomeDbBasicForm *form, GdaParameter *para
  * Returns: the new #GtkDialog widget
  */
 GtkWidget *
-gnome_db_basic_form_new_in_dialog (GdaParameterList *paramlist, GtkWindow *parent,
+gnome_db_basic_form_new_in_dialog (GdaSet *paramlist, GtkWindow *parent,
 				   const gchar *title, const gchar *header)
 {
 	GtkWidget *form;
@@ -1640,7 +1691,7 @@ gnome_db_basic_form_new_in_dialog (GdaParameterList *paramlist, GtkWindow *paren
 }
 
 static void
-form_param_changed (GnomeDbBasicForm *form, GdaParameter *param, gboolean is_user_modif, GtkDialog *dlg)
+form_param_changed (GnomeDbBasicForm *form, GdaHolder *param, gboolean is_user_modif, GtkDialog *dlg)
 {
 	gboolean valid;
 
