@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2005 The GNOME Foundation.
+ * Copyright (C) 1999 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
@@ -21,8 +21,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if !defined(__gnome_db_provider_selector_h__)
-#  define __gnome_db_provider_selector_h__
+#ifndef __GNOME_DB_PROVIDER_SELECTOR_H__
+#define __GNOME_DB_PROVIDER_SELECTOR_H__
 
 #include <libgnomedb/gnome-db-combo.h>
 
@@ -47,12 +47,12 @@ struct _GnomeDbProviderSelectorClass {
 	GnomeDbComboClass               parent_class;
 };
 
-GType              gnome_db_provider_selector_get_type                  (void);
-GtkWidget         *gnome_db_provider_selector_new                       (void);
-GdaServerProvider *gnome_db_provider_selector_get_selected_provider_obj (GnomeDbProviderSelector *selector);
-const gchar       *gnome_db_provider_selector_get_selected_provider     (GnomeDbProviderSelector *selector);
-gboolean           gnome_db_provider_selector_set_selected_provider     (GnomeDbProviderSelector *selector, 
-									 const gchar *provider);
+GType              gnome_db_provider_selector_get_type         (void) G_GNUC_CONST;
+GtkWidget         *gnome_db_provider_selector_new              (void);
+
+GdaServerProvider *gnome_db_provider_selector_get_provider_obj (GnomeDbProviderSelector *selector);
+const gchar       *gnome_db_provider_selector_get_provider     (GnomeDbProviderSelector *selector);
+gboolean           gnome_db_provider_selector_set_provider     (GnomeDbProviderSelector *selector, const gchar *provider);
 
 G_END_DECLS
 
