@@ -2,17 +2,17 @@
  *
  * Copyright (C) 2002 - 2008 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Library General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -51,8 +51,9 @@ static void proxy_sample_changed_cb (GdaDataProxy *proxy, gint sample_start, gin
 static void proxy_row_updated_cb (GdaDataProxy *proxy, gint proxy_row, GnomeDbRawGrid *grid);
 static void proxy_reset_cb (GdaDataProxy *proxy, GnomeDbRawGrid *grid);
 static void paramlist_public_data_changed_cb (GdaSet *paramlist, GnomeDbRawGrid *grid);
-static void paramlist_param_plugin_changed_cb (GdaSet *paramlist, GdaHolder *param, GnomeDbRawGrid *grid);
-static gboolean iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawGrid *grid);
+static void paramlist_param_attr_changed_cb (GdaSet *paramlist, GdaHolder *param, 
+					     const gchar *att_name, const GValue *att_value, GnomeDbRawGrid *grid);
+static GError *iter_validate_set_cb (GdaDataModelIter *iter, GnomeDbRawGrid *grid);
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawGrid *grid);
 
 /* GnomeDbDataWidget interface */
@@ -393,13 +394,13 @@ gnome_db_raw_grid_set_property (GObject *object,
 				
 				g_signal_connect (grid->priv->iter, "public_data_changed",
 						  G_CALLBACK (paramlist_public_data_changed_cb), grid);
-				g_signal_connect (grid->priv->iter, "holder_plugin_changed",
-						  G_CALLBACK (paramlist_param_plugin_changed_cb), grid);
+				g_signal_connect (grid->priv->iter, "holder-attr-changed",
+						  G_CALLBACK (paramlist_param_attr_changed_cb), grid);
 
 				g_signal_connect (grid->priv->iter, "row_changed",
 						  G_CALLBACK (iter_row_changed_cb), grid);
-				g_signal_connect (grid->priv->iter, "row_to_change",
-						  G_CALLBACK (iter_row_to_change_cb), grid);
+				g_signal_connect (grid->priv->iter, "validate-set",
+						  G_CALLBACK (iter_validate_set_cb), grid);
 
 				gda_data_model_iter_invalidate_contents (grid->priv->iter);
 							
@@ -626,7 +627,8 @@ init_tree_view (GnomeDbRawGrid *grid)
 		else {
 			/* single direct parameter */
 			GType g_type;
-			gchar *plugin = NULL;
+			const gchar *plugin = NULL;
+			const GValue *plugin_val;
 			gchar *title;
 			
 			param = GDA_HOLDER (GDA_SET_NODE (group->nodes->data)->holder);
@@ -640,7 +642,14 @@ init_tree_view (GnomeDbRawGrid *grid)
 			if (!title)
 				title = g_strdup (_("No title"));
 			
-			g_object_get (G_OBJECT (param), "plugin", &plugin, NULL);
+			plugin_val = gda_holder_get_attribute (param, GNOME_DB_ATTRIBUTE_PLUGIN);
+			if (plugin_val) {
+				if (G_VALUE_TYPE (plugin_val) == G_TYPE_STRING)
+					plugin = g_value_get_string (plugin_val);
+				else
+					g_warning (_("The '%s' attribute should be a G_TYPE_STRING value"),
+						   GNOME_DB_ATTRIBUTE_PLUGIN);
+			}
 			renderer = gnome_db_util_new_cell_renderer (g_type, plugin);
 			column_data->data_cell = renderer;
 			if (! column_data->data_locked)
@@ -1158,7 +1167,7 @@ action_commit_cb (GtkAction *action, GnomeDbRawGrid *grid)
 					      actually succedded the commit 
 					      => we need to come back to that row
 					   */
-				gda_data_model_iter_set_at_row (grid->priv->iter, row);
+				gda_data_model_iter_move_at_row (grid->priv->iter, row);
 		}
 	}
 	else
@@ -1880,7 +1889,7 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbRawGrid *gri
 		has_selection = gtk_tree_selection_get_selected (selection, &model, &iter) ? 1 : 0;
 
 	if (has_selection == 1) {
-		if (!gda_data_model_iter_set_at_row (grid->priv->iter, 
+		if (!gda_data_model_iter_move_at_row (grid->priv->iter, 
 						     gnome_db_data_store_get_row_from_iter (grid->priv->store, 
 											    &iter))) {
 			/* selection changing is refused, return to the current selected row */
@@ -1895,7 +1904,7 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbRawGrid *gri
 	else {
 		/* render all the parameters invalid, and make the iter point to row -1 */
 		gda_data_model_iter_invalidate_contents (grid->priv->iter);
-		gda_data_model_iter_set_at_row (grid->priv->iter, -1);
+		gda_data_model_iter_move_at_row (grid->priv->iter, -1);
 	}
 
 #ifdef debug_signal
@@ -2143,33 +2152,39 @@ paramlist_public_data_changed_cb (GdaSet *paramlist, GnomeDbRawGrid *grid)
 }
 
 static void
-paramlist_param_plugin_changed_cb (GdaSet *paramlist, GdaHolder *param, GnomeDbRawGrid *grid)
+paramlist_param_attr_changed_cb (GdaSet *paramlist, GdaHolder *param, 
+				 const gchar *att_name, const GValue *att_value, GnomeDbRawGrid *grid)
 {
-	/* TODO: be more specific and change only the cell renderer corresponding to @param */
-	paramlist_public_data_changed_cb (paramlist, grid);
+	if (!strcmp (att_name, GNOME_DB_ATTRIBUTE_PLUGIN)) {
+		/* TODO: be more specific and change only the cell renderer corresponding to @param */
+		paramlist_public_data_changed_cb (paramlist, grid);
+	}
 }
 
-static gboolean
-iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawGrid *grid)
+static GError *
+iter_validate_set_cb (GdaDataModelIter *iter, GnomeDbRawGrid *grid)
 {
-	gboolean allow_change = TRUE;
-	if (row >= 0) {
-		if (grid->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) {
-			/* write back the current row */
-			if (gda_data_proxy_row_has_changed (grid->priv->proxy, row)) {
-				GError *error = NULL;
-				if (!gda_data_proxy_apply_row_changes (grid->priv->proxy, row, &error)) {
-					allow_change = gnome_db_utility_display_error_with_keep_or_discard_choice ((GnomeDbDataWidget *) grid, 
-													  error);
-					if (allow_change)
-						gda_data_proxy_cancel_row_changes (grid->priv->proxy, row, -1);
-					g_error_free (error);
-				}
+GError *error = NULL;
+	gint row = gda_data_model_iter_get_row (iter);
+	
+	if (row < 0)
+		return NULL;
+
+	if ((grid->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) &&
+		/* write back the current row */
+	    gda_data_proxy_row_has_changed (grid->priv->proxy, row) &&
+	    !gda_data_proxy_apply_row_changes (grid->priv->proxy, row, &error)) {
+		if (gnome_db_utility_display_error_with_keep_or_discard_choice ((GnomeDbDataWidget *) grid, 
+										error)) {
+			gda_data_proxy_cancel_row_changes (grid->priv->proxy, row, -1);
+			if (error) {
+				g_error_free (error);
+				error = NULL;
 			}
 		}
 	}
 	
-	return allow_change;
+	return error;	
 }
 
 static void
@@ -2292,11 +2307,11 @@ gnome_db_raw_grid_clean (GnomeDbRawGrid *grid)
 		g_signal_handlers_disconnect_by_func (grid->priv->iter,
 						      G_CALLBACK (paramlist_public_data_changed_cb), grid);
 		g_signal_handlers_disconnect_by_func (grid->priv->iter,
-						      G_CALLBACK (paramlist_param_plugin_changed_cb), grid);
+						      G_CALLBACK (paramlist_param_attr_changed_cb), grid);
 		g_signal_handlers_disconnect_by_func (grid->priv->iter,
 						      G_CALLBACK (iter_row_changed_cb), grid);
 		g_signal_handlers_disconnect_by_func (grid->priv->iter,
-						      G_CALLBACK (iter_row_to_change_cb), grid);
+						      G_CALLBACK (iter_validate_set_cb), grid);
 		g_object_unref (grid->priv->iter);
 		grid->priv->iter = NULL;
 	}
