@@ -1,18 +1,18 @@
 /* gnome-db-server-operation.c
  *
- * Copyright (C) 2006 - 2007 Vivien Malerba
+ * Copyright (C) 2006 - 2008 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Library General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -29,6 +29,7 @@
 #include "gnome-db-data-store.h"
 #include "gnome-db-data-widget.h"
 #include "gnome-db-data-widget-info.h"
+#include <libgnomedb/binreloc/gnome-db-binreloc.h>
 
 static void gnome_db_server_operation_class_init (GnomeDbServerOperationClass *class);
 static void gnome_db_server_operation_init (GnomeDbServerOperation *wid);
@@ -66,7 +67,9 @@ struct _GnomeDbServerOperationPriv
 	GdaServerOperation     *op;
 	GSList                 *widget_data; /* list of WidgetData structures */
 	GtkTooltips            *tooltips;
+#ifdef HAVE_LIBGLADE
 	GladeXML               *glade;
+#endif
 	gboolean                opt_header;
 };
 
@@ -159,7 +162,7 @@ gnome_db_server_operation_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbServerOperationClass),
 			(GBaseInitFunc) NULL,
@@ -212,14 +215,16 @@ gnome_db_server_operation_init (GnomeDbServerOperation * wid)
 	wid->priv->tooltips = gtk_tooltips_new ();
 	g_object_ref (G_OBJECT (wid->priv->tooltips));
 	gtk_object_sink (GTK_OBJECT (wid->priv->tooltips));
+#ifdef HAVE_LIBGLADE
 	wid->priv->glade = NULL;
+#endif
 	wid->priv->opt_header = FALSE;
 }
 
 
 /**
  * gnome_db_server_operation_new
- * @paramlist: a #GdaParameterList structure
+ * @paramlist: a #GdaSet structure
  *
  * Creates a new #GnomeDbServerOperation widget using all the parameters provided in @paramlist.
  *
@@ -267,8 +272,10 @@ gnome_db_server_operation_dispose (GObject *object)
 		}
 		
 		g_object_unref (G_OBJECT (form->priv->tooltips));
+#ifdef HAVE_LIBGLADE
 		if (form->priv->glade)
 			g_object_unref (form->priv->glade);
+#endif
 
 		/* the private area itself */
 		g_free (form->priv);
@@ -414,7 +421,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 	/* generic widget rendering */
 	switch (info_node->type) {
 	case GDA_SERVER_OPERATION_NODE_PARAMLIST: {
-		GdaParameterList *plist;
+		GdaSet *plist;
 
 		plist = info_node->plist;
 		plwid = gnome_db_basic_form_new (plist);
@@ -423,7 +430,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 	       
 		if (section_str) {
 			const gchar *name;
-			name = gda_object_get_name (GDA_OBJECT (plist));
+			name = g_object_get_data (G_OBJECT (plist), "name");
 			if (name && *name)
 				*section_str = g_strdup_printf ("<b>%s:</b>", name);
 			else
@@ -432,12 +439,12 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 		if (label_widgets) {
 			GSList *params;
 			
-			params = plist->parameters;
+			params = plist->holders;
 			while (params) {
 				GtkWidget *label_entry;
 
 				label_entry = gnome_db_basic_form_get_label_widget (GNOME_DB_BASIC_FORM (plwid),
-										    GDA_PARAMETER (params->data));
+										    GDA_HOLDER (params->data));
 				if (label_entry && !g_slist_find (*label_widgets, label_entry))
 					*label_widgets = g_slist_prepend (*label_widgets, label_entry);
 				params = params->next;
@@ -484,7 +491,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 		
 		if (section_str)
 			*section_str = g_strdup_printf ("<b>%s:</b>", 
-						       gda_object_get_name (GDA_OBJECT (model)));
+							(gchar*) g_object_get_data (G_OBJECT (model), "name"));
 
 		if (label_widgets) {
 			GtkWidget *label_entry;
@@ -492,12 +499,13 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 			gchar *str;
 
 			if (info_node->status == GDA_SERVER_OPERATION_STATUS_REQUIRED) {
-				str = g_strdup_printf ("<b>%s:</b>", gda_object_get_name (GDA_OBJECT (model)));
+				str = g_strdup_printf ("<b>%s:</b>", 
+						       (gchar*) g_object_get_data (G_OBJECT (model), "name"));
 				label_entry = gtk_label_new (str);
 				gtk_label_set_use_markup (GTK_LABEL (label_entry), TRUE);
 			}
 			else {
-				str = g_strdup_printf ("%s:", gda_object_get_name (GDA_OBJECT (model)));
+				str = g_strdup_printf ("%s:", (gchar*) g_object_get_data (G_OBJECT (model), "name"));
 				label_entry = gtk_label_new (str);
 			}
 			g_free (str);
@@ -506,7 +514,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 			evbox = gtk_event_box_new ();
 			gtk_container_add (GTK_CONTAINER (evbox), label_entry);
 			gtk_widget_show (label_entry);
-			str = (gchar *) gda_object_get_description (GDA_OBJECT (model));
+			str = (gchar *) g_object_get_data (G_OBJECT (model), "descr");
 			if (str && *str) 
 				gtk_tooltips_set_tip (form->priv->tooltips, evbox, str, NULL);
 
@@ -518,13 +526,13 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 		break;
 	}
 	case GDA_SERVER_OPERATION_NODE_PARAM: {
-		GdaParameterList *plist;
-		GdaParameter *param;
+		GdaSet *plist;
+		GdaHolder *param;
 		GSList *list;
 
 		param = info_node->param;
 		list = g_slist_append (NULL, param);
-		plist = gda_parameter_list_new (list);
+		plist = gda_set_new (list);
 		g_slist_free (list);
 		plwid = gnome_db_basic_form_new (plist);
 		gnome_db_basic_form_show_entry_actions (GNOME_DB_BASIC_FORM (plwid),
@@ -534,7 +542,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 
 		if (section_str)
 			*section_str = g_strdup_printf ("<b>%s:</b>", 
-							gda_object_get_name (GDA_OBJECT (param)));
+							(gchar*) g_object_get_data (G_OBJECT (param), "name"));
 		if (label_widgets) {
 			GtkWidget *label_entry;
 
@@ -712,7 +720,8 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 		return;
 
 	/* load Glade file for specific GUI if it exists */
-	glade_file = g_build_filename (DATADIR, "gnome-db", "server_operation.glade", NULL);
+#ifdef HAVE_LIBGLADE
+	glade_file = gnome_db_gbr_get_data_dir_path ("server_operation.glade");
 	form->priv->glade = glade_xml_new (glade_file, 
 					   gda_server_operation_op_type_to_string (gda_server_operation_get_op_type (form->priv->op)), 
 					   NULL);
@@ -730,6 +739,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 			form->priv->glade = NULL;
 		}
 	}
+#endif
 
 	/* user visible widgets */
 	topnodes = gda_server_operation_get_root_nodes (form->priv->op);
@@ -739,6 +749,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 		gchar *section_str;
 		GtkWidget *container = NULL;
 
+#ifdef HAVE_LIBGLADE
 		if (form->priv->glade) {
 			container = glade_xml_get_widget (form->priv->glade, topnodes[i]);
 			if (!container) {
@@ -746,6 +757,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 				continue;
 			}
 		}
+#endif
 		if (!container)
 			container = (GtkWidget *) form;
 
@@ -820,6 +832,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 	}
 
 	/* destroying unused widgets in the Glade description */
+#ifdef HAVE_LIBGLADE
 	if (form->priv->glade) {
 		GList *widgets, *list;
 
@@ -850,6 +863,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 		}
 		g_list_free (widgets);
 	}
+#endif
 
 	g_strfreev (topnodes);
 	
@@ -1231,7 +1245,7 @@ create_table_grid_fields_iter_row_changed_cb (GdaDataModelIter *iter1, gint row,
 {
 	g_signal_handlers_block_by_func (G_OBJECT (iter2),
 					 G_CALLBACK (create_table_grid_fields_iter_row_changed_cb), iter1);
-	gda_data_model_iter_set_at_row (iter2, row);
+	gda_data_model_iter_move_at_row (iter2, row);
 	g_signal_handlers_unblock_by_func (G_OBJECT (iter2),
 					   G_CALLBACK (create_table_grid_fields_iter_row_changed_cb), iter1);
 }
